import unittest
from mock import MagicMock
from allensdk.api.queries.grid_data_api import GridDataApi

class GridDataApiTests(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(GridDataApiTests, self).__init__(*args, **kwargs)
        self.ida = None
    
    
    def setUp(self):
        self.gda = GridDataApi()
    
    
    def tearDown(self):
        self.ida = None
    
    
    def test_api_doc_url_download_expression_grid(self):
        '''Url to download the 200um density volume for the Mouse Brain Atlas SectionDataSet 69816930.
        
        Notes
        -----
        See `Downloading 3-D Expression Grid Data <http://help.brain-map.org/display/api/Downloading+3-D+Expression+Grid+Data#Downloading3-DExpressionGridData-DOWNLOADING3DEXPRESSIONGRIDDATA>`_
        , example 'Download the 200um density volume for the Mouse Brain Atlas SectionDataSet 69816930'.
        '''
        expected = 'http://api.brain-map.org/grid_data/download/69816930'
        path = '69816930.zip'
        
        section_data_set_id = 69816930
        
        self.gda.retrieve_file_over_http = \
            MagicMock(name='retrieve_file_over_http')
        
        self.gda.download_expression_grid_data(section_data_set_id)
        
        self.gda.retrieve_file_over_http.assert_called_once_with(expected, path)
    
    
    def test_api_doc_url_download_expression_grid_energy_intensity(self):
        '''Url to download the 200um energy and intensity volumes for Mouse Brain Atlas SectionDataSet 69816930.
        
        Notes
        -----
        See `Downloading 3-D Expression Grid Data <http://help.brain-map.org/display/api/Downloading+3-D+Expression+Grid+Data#Downloading3-DExpressionGridData-DOWNLOADING3DEXPRESSIONGRIDDATA>`_
        , example 'Download the 200um energy and intensity volumes for Mouse Brain Atlas SectionDataSet 69816930'.
        
        The id in the example url doesn't match the caption.
        '''
        expected = 'http://api.brain-map.org/grid_data/download/183282970?include=energy,intensity'
        path = '183282970.zip'

        section_data_set_id = 183282970
        include = ['energy', 'intensity']
        
        self.gda.retrieve_file_over_http = \
            MagicMock(name='retrieve_file_over_http')
        
        self.gda.download_expression_grid_data(section_data_set_id,
                                               include=include)
        
        self.gda.retrieve_file_over_http.assert_called_once_with(expected, path)
    
    
    def test_api_doc_url_projection_grid(self):
        '''Url to download the 100um density volume for the Mouse Connectivity Atlas SectionDataSet 181777177.
        
        Notes
        -----
        See `Downloading 3-D Projection Grid Data <http://help.brain-map.org/display/api/Downloading+3-D+Expression+Grid+Data#Downloading3-DExpressionGridData-DOWNLOADING3DPROJECTIONGRIDDATA>`_
        , example 'Download the 100um density volume for the Mouse Connectivity Atlas SectionDataSet 181777177'.
        '''
        expected = 'http://api.brain-map.org/grid_data/download_file/181777177'
        path = '181777177.nrrd'

        self.gda.retrieve_file_over_http = \
            MagicMock(name='retrieve_file_over_http')
        
        section_data_set_id = 181777177
        self.gda.download_projection_grid_data(section_data_set_id)
        
        self.gda.retrieve_file_over_http.assert_called_once_with(expected, path)
    
    
    def test_api_doc_url_projection_grid_injection_fraction_resolution(self):
        '''Url to download the 25um injection_fraction volume for Mouse Connectivity Atlas SectionDataSet 181777177.
        
        Notes
        -----
        See `Downloading 3-D Projection Grid Data <http://help.brain-map.org/display/api/Downloading+3-D+Expression+Grid+Data#Downloading3-DExpressionGridData-DOWNLOADING3DPROJECTIONGRIDDATA>`_
        , example 'Download the 25um injection_fraction volume for Mouse Connectivity Atlas SectionDataSet 181777177'.
        '''
        expected = 'http://api.brain-map.org/grid_data/download_file/181777177?image=injection_fraction&resolution=25'

        self.gda.retrieve_file_over_http = \
            MagicMock(name='retrieve_file_over_http')
        
        section_data_set_id = 181777177
        path = 'id.nrrd'
        self.gda.download_projection_grid_data(section_data_set_id,
                                               [self.gda.INJECTION_FRACTION],
                                               resolution=25,
                                               save_file_path=path)
        
        self.gda.retrieve_file_over_http.assert_called_once_with(expected, path)


# Copyright 2015 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.

import unittest
from mock import MagicMock
from allensdk.model.biophys_sim.config import Config
from allensdk.core.json_utilities import JsonComments

class ConfigSingleFileJsonTests(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(ConfigSingleFileJsonTests, self).__init__(*args, **kwargs)
    
    def setUp(self):
        manifest = {
            'manifest': [
                { 'type': 'dir',
                  'spec': 'MOCK_DOT',
                  'key' : 'BASEDIR'
                }],
            'biophys':
                [{ 'hoc': [ 'stdgui.hoc'] }],
        }

        ju = JsonComments
        ju.read_file = MagicMock(return_value=manifest)
    
    
    def tearDown(self):
        pass
    
    def testAccessHocFilesInData(self):
        config = Config().load('config.json', False)
        self.assertEqual(config.data['biophys'][0]['hoc'][0],
                         'stdgui.hoc')
    
    def testManifestIsNotInData(self):
        config = Config().load('config.json', False)
        self.assertFalse('manifest' in config.data)
    
    
    def testManifestInReservedData(self):
        config = Config().load('config.json', False)
        self.assertTrue('manifest' in config.reserved_data[0])  # huh, why [0]?


# Copyright 2015 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.

import unittest
from mock import patch, mock_open
from allensdk.core.json_utilities import JsonComments

class JsonCommentsTest(unittest.TestCase):
    commented_json = ("{\n"
                      "    // comment\n"
                      "    \"color\": \"blue\"\n"
                      "}")
    
    blank_line_json = ("{\n"
                      "\n"
                      "\n"
                      "\n"
                      "    \"color\": \"blue\"\n"
                      "}")
    
    multi_line_json = ("{\n"
                      "/* \n"
                      " * multiline comment\n"
                      " */\n"
                      "    \"color\": \"blue\"\n"
                      "}")
    
    two_multi_line_json = ("{\n"
                      "    \"colors\": [\"blue\",\n"
                      "    /* comment these out\n"
                      "    \"red\",\n"
                      "    \"yellow\",\n"
                      "     ... but not these */\n"
                      "    \"orange\",\n"
                      "    \"purple\",\n"
                      "    /* also comment this out\n"
                      "    \"indigo\",\n"
                      "    .... end comment */\n"
                      "    \"violet\"\n"
                      "    ]\n"
                      "}")
    
    
    def __init__(self, *args, **kwargs):
        super(JsonCommentsTest, self).__init__(*args, **kwargs)
    
    
    def setUp(self):
        pass
    
    
    def tearDown(self):
        pass
    
    
    def testSingleLineComment(self):
        parsed_json = JsonComments.read_string(
            JsonCommentsTest.commented_json)
        
        self.assertTrue('color' in parsed_json and
                        parsed_json['color'] == 'blue')

        
    def testBlankLines(self):
        parsed_json = JsonComments.read_string(
            JsonCommentsTest.blank_line_json)
        
        self.assertTrue('color' in parsed_json and
                        parsed_json['color'] == 'blue')


    def testMultiLineComment(self):
        parsed_json = JsonComments.read_string(
            JsonCommentsTest.multi_line_json)
        
        self.assertTrue('color' in parsed_json and
                        parsed_json['color'] == 'blue')

    def testTwoMultiLineComments(self):
        parsed_json = JsonComments.read_string(
            JsonCommentsTest.two_multi_line_json)
        
        self.assertTrue('colors' in parsed_json)
        self.assertTrue(len(parsed_json['colors']) == 4)
        self.assertTrue('blue' in parsed_json['colors'])
        self.assertTrue('orange' in parsed_json['colors'])
        self.assertTrue('purple' in parsed_json['colors'])
        self.assertTrue('violet' in parsed_json['colors'])
    
    
    def testSingleLineCommentFile(self):
        with patch("__builtin__.open",
                   mock_open(
                       read_data=JsonCommentsTest.commented_json)):
            parsed_json = JsonComments.read_file('mock.json')
        
        self.assertTrue('color' in parsed_json and
                        parsed_json['color'] == 'blue')
    
    
    def testBlankLinesFile(self):
        with patch("__builtin__.open",
                   mock_open(
                       read_data=JsonCommentsTest.blank_line_json)):
            parsed_json = JsonComments.read_file('mock.json')
        
        self.assertTrue('color' in parsed_json and
                        parsed_json['color'] == 'blue')
    
    
    def testMultiLineFile(self):
        with patch("__builtin__.open",
                   mock_open(
                       read_data=JsonCommentsTest.multi_line_json)):
            parsed_json = JsonComments.read_file('mock.json')
        
        self.assertTrue('color' in parsed_json and
                        parsed_json['color'] == 'blue')
    
    
    def testTwoMultiLineFile(self):
        with patch("__builtin__.open",
                   mock_open(
                       read_data=JsonCommentsTest.two_multi_line_json)):
            parsed_json = JsonComments.read_file('mock.json')
        
        self.assertTrue('colors' in parsed_json)
        self.assertTrue(len(parsed_json['colors']) == 4)
        self.assertTrue('blue' in parsed_json['colors'])
        self.assertTrue('orange' in parsed_json['colors'])
        self.assertTrue('purple' in parsed_json['colors'])
        self.assertTrue('violet' in parsed_json['colors'])



# Copyright 2015 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.

from allensdk.api.api import Api

class RmaApi(Api):
    '''
    See: `RESTful Model Access (RMA) <http://help.brain-map.org/display/api/RESTful+Model+Access+%28RMA%29>`_
    '''
    MODEL='model::'
    PIPE='pipe::'
    SERVICE='service::'
    CRITERIA='rma::criteria'
    INCLUDE='rma::include'
    OPTIONS='rma::options'
    ORDER='order'
    NUM_ROWS='num_rows'
    ALL='all'
    START_ROW='start_row'
    COUNT='count'
    ONLY='only'
    EXCEPT='except'
    TABULAR='tabular'
    DEBUG='debug'
    PREVIEW='preview'
    TRUE='true'
    FALSE='false'
    IS='$is'
    EQ='$eq'
    
    def __init__(self, base_uri=None):
        super(RmaApi, self).__init__(base_uri)
    
    
    def build_query_url(self,
                        stage_clauses,
                        fmt='json'):
        '''Combine one or more RMA query stages into a single RMA query.
        
        Parameters
        ----------
        stage_clauses : list of strings
            subqueries
        fmt : string, optional
            json (default), xml, or csv
            
        Returns
        -------
        string
            complete RMA url
        '''
        if not type(stage_clauses) is list:
            stage_clauses = [stage_clauses]
        
        url = ''.join([
            self.rma_endpoint,
            '/query.',
            fmt,
            '?q=',
            ','.join(stage_clauses)])
        
        return url
    
    def model_stage(self,
                    model,
                    **kwargs):
        '''Construct a model stage of an RMA query string.
        
        Parameters
        ----------
        model : string
            The top level data type
        filters : dict
            key, value comparisons applied to the top-level model to narrow the results.            
        criteria : string
            raw RMA criteria clause to choose what object are returned
        include : string
            raw RMA include clause to return associated objects
        only : list of strings, optional
            to be joined into an rma::options only filter to limit what data is returned
        except : list of strings, optional
            to be joined into an rma::options except filter to limit what data is returned            
        tabular : list of string, optional
            return columns as a tabular data structure rather than a nested tree.                    
        count : boolean, optional
            False to skip the extra database count query.                    
        debug : string, optional
            'true', 'false' or 'preview'
        num_rows : int or string, optional
            how many database rows are returned (may not correspond directly to JSON tree structure) 
        start_row : int or string, optional
            which database row is start of returned data  (may not correspond directly to JSON tree structure)
        
        
        Notes
        -----
        See `RMA Path Syntax <http://help.brain-map.org/display/api/RMA+Path+Syntax#RMAPathSyntax-DoubleColonforAxis>`_
        for a brief overview of the normalized RMA syntax.
        Normalized RMA syntax differs from the legacy syntax
        used in much of the RMA documentation.
        Using the &debug=true option with an RMA URL will include debugging information in the
        response, including the normalized query.
        '''
        clauses = [RmaApi.MODEL + model]
        
        filters = kwargs.get('filters', None)
        
        if filters != None:
            clauses.append(self.filters(filters))
            
        criteria = kwargs.get('criteria', None)
        
        if criteria != None:
            clauses.append(',')
            clauses.append(RmaApi.CRITERIA)
            clauses.append(',')
            clauses.extend(criteria)
        
        include = kwargs.get('include', None)
        
        if include != None:
            clauses.append(',')
            clauses.append(RmaApi.INCLUDE)
            clauses.append(',')
            clauses.extend(include)
        
        options_clause = self.options_clause(**kwargs)
        
        if options_clause != '':
            clauses.append(',')
            clauses.append(options_clause)
        
        stage = ''.join(clauses)
        
        return stage
    
    
    def pipe_stage(self,
                   pipe_name,
                   parameters):
        '''Connect model and service stages via their JSON responses.
        
        Notes
        -----
        See: `Service Pipelines <http://help.brain-map.org/display/api/Service+Pipelines>`_ 
        and
        `Connected Services and Pipes <http://help.brain-map.org/display/api/Connected+Services+and+Pipes>`_
        '''
        clauses = [RmaApi.PIPE + pipe_name]
        
        clauses.append(self.tuple_filters(parameters))
        
        stage = ''.join(clauses)
        
        return stage
    
    
    def service_stage(self,
                      service_name,
                      parameters=None):
        '''Construct an RMA query fragment to send a request to a connected service.
        
        Parameters
        ----------
        service_name : string
            Name of a documented connected service.
        parameters : dict
            key-value pairs as in the online documentation.
        
        Notes
        -----
        See: `Service Pipelines <http://help.brain-map.org/display/api/Service+Pipelines>`_ 
        and
        `Connected Services and Pipes <http://help.brain-map.org/display/api/Connected+Services+and+Pipes>`_
        '''
        clauses = [RmaApi.SERVICE + service_name]
        
        if parameters != None:
            clauses.append(self.tuple_filters(parameters))
        
        stage = ''.join(clauses)

        return stage


    def model_query(self, *args, **kwargs):
        '''Construct and execute a model stage of an RMA query string.
        
        Parameters
        ----------
        model : string
            The top level data type
        filters : dict
            key, value comparisons applied to the top-level model to narrow the results.            
        criteria : string
            raw RMA criteria clause to choose what object are returned
        include : string
            raw RMA include clause to return associated objects
        only : list of strings, optional
            to be joined into an rma::options only filter to limit what data is returned
        except : list of strings, optional
            to be joined into an rma::options except filter to limit what data is returned            
        tabular : list of string, optional
            return columns as a tabular data structure rather than a nested tree.                    
        count : boolean, optional
            False to skip the extra database count query.                    
        debug : string, optional
            'true', 'false' or 'preview'
        num_rows : int or string, optional
            how many database rows are returned (may not correspond directly to JSON tree structure) 
        start_row : int or string, optional
            which database row is start of returned data  (may not correspond directly to JSON tree structure)
        
        
        Notes
        -----
        See `RMA Path Syntax <http://help.brain-map.org/display/api/RMA+Path+Syntax#RMAPathSyntax-DoubleColonforAxis>`_
        for a brief overview of the normalized RMA syntax.
        Normalized RMA syntax differs from the legacy syntax
        used in much of the RMA documentation.
        Using the &debug=true option with an RMA URL will include debugging information in the
        response, including the normalized query.
        '''
        return self.json_msg_query(
            self.build_query_url(
                self.model_stage(*args, **kwargs)))
    
    
    def service_query(self, *args, **kwargs):
        '''Construct and Execute a single-stage RMA query
        to send a request to a connected service.
        
        Parameters
        ----------
        service_name : string
            Name of a documented connected service.
        parameters : dict
            key-value pairs as in the online documentation.
        
        Notes
        -----
        See: `Service Pipelines <http://help.brain-map.org/display/api/Service+Pipelines>`_ 
        and
        `Connected Services and Pipes <http://help.brain-map.org/display/api/Connected+Services+and+Pipes>`_
        '''        
        return self.json_msg_query(
            self.build_query_url(
                self.service_stage(*args, **kwargs)))
    
    
    def options_clause(self, **kwargs):
        '''build rma:: options clause.
        
        Parameters
        ----------
        only : list of strings, optional
        except : list of strings, optional
        tabular : list of string, optional
        count : boolean, optional
        debug : string, optional
            'true', 'false' or 'preview'
        num_rows : int or string, optional
        start_row : int or string, optional
        '''
        clause = ''
        options_params = []
        
        only = kwargs.get(RmaApi.ONLY, None)
        
        if only != None:
            options_params.append(
                self.only_except_tabular_clause(RmaApi.ONLY,
                                                only))
        
        excpt = kwargs.get(RmaApi.EXCEPT, None)
        
        if excpt != None:
            options_params.append(
                self.only_except_tabular_clause(RmaApi.EXCEPT,
                                                excpt))
        
        tabular = kwargs.get(RmaApi.TABULAR, None)
        
        if tabular != None:
            options_params.append(
                self.only_except_tabular_clause(RmaApi.TABULAR,
                                                tabular))
        
        num_rows = kwargs.get(RmaApi.NUM_ROWS, None)
        
        if num_rows != None:
            if num_rows == RmaApi.ALL:
                options_params.append("[%s$eq'all']" % (RmaApi.NUM_ROWS))
            else:
                options_params.append('[%s$eq%d]' % (RmaApi.NUM_ROWS,
                                                     num_rows))
        
        start_row = kwargs.get(RmaApi.START_ROW, None)
        
        if start_row != None:
            options_params.append('[%s$eq%d]' % (RmaApi.START_ROW,
                                                 start_row))
        
        order = kwargs.get(RmaApi.ORDER, None)
        
        if order != None:
            options_params.append(self.order_clause(order))
        
        debug = kwargs.get(RmaApi.DEBUG, None)
        
        if debug != None:
            options_params.append(self.debug_clause(debug))
        
        cnt = kwargs.get(RmaApi.COUNT, None)
        
        if cnt != None:
            if cnt == True or cnt == 'true':
                options_params.append('[%s$eq%s]' % (RmaApi.COUNT,
                                                     RmaApi.TRUE))
            elif cnt == False or cnt == 'false':
                options_params.append('[%s$eq%s]' % (RmaApi.COUNT,
                                                     RmaApi.FALSE))
            else:
                pass
        
        if len(options_params) > 0:
            clause = RmaApi.OPTIONS + ''.join(options_params)
        
        return clause
    
    
    def only_except_tabular_clause(self, filter_type, attribute_list):
        '''Construct a clause to filter which attributes are returned
        for use in an rma::options clause.
        
        Parameters
        ----------
        filter_type : string
            'only', 'except', or 'tabular'
        attribute_list : list of strings
            for example ['acronym', 'products.name', 'structure.id']
        
        Returns
        -------
        clause : string
            The query clause for inclusion in an RMA query URL.
        
        Notes
        -----
        The title of tabular columns can be set by adding '+as+<title>'
        to the attribute.
        The tabular filter type requests a response that is row-oriented
        rather than a nested structure.
        Because of this, the tabular option can mask the lazy query behavior
        of an rma::include clause.
        The tabular option does not mask the inner-join behavior of an rma::include
        clause.
        The tabular filter is required for .csv format RMA requests.
        '''
        clause = ''
        
        if attribute_list != None:
            clause = '[%s$eq%s]' % (filter_type,
                                    ','.join(attribute_list))
        
        return clause
    
    
    def order_clause(self, order_list=None):
        '''Construct a debug clause for use in an rma::options clause.

        Parameters
        ----------
        order_list : list of strings
            for example ['acronym', 'products.name+asc', 'structure.id+desc']
        
        Returns
        -------
        clause : string
            The query clause for inclusion in an RMA query URL.
        
        Notes
        -----
        Optionally adding '+asc' (default) or '+desc' after an attribute
        will change the sort order.
        '''
        clause = ''
        
        if order_list != None:
            clause = '[order$eq%s]' % (','.join(order_list))
        
        return clause
    
    
    def debug_clause(self, debug_value=None):
        '''Construct a debug clause for use in an rma::options clause.
        Parameters
        ----------
        debug_value : string or boolean
            True, False, None (default) or 'preview'
        
        Returns
        -------
        clause : string
            The query clause for inclusion in an RMA query URL.
        
        Notes
        -----
        True will request debugging information in the response.
        False will request no debugging information.
        None will return an empty clause.
        'preview' will request debugging information without the query being run.
         
        '''
        clause = ''
        
        if debug_value == None:
            clause = ''
        if debug_value == True or debug_value == 'true':
            clause = '[debug$eqtrue]'
        elif debug_value == False or debug_value == 'false':
            clause = '[debug$eqfalse]'
        elif debug_value == 'preview':
            clause = "[debug$eq'preview']"
        
        return clause
    
    
    # TODO: deprecate for something that can preserve order
    def filters(self, filters):
        '''serialize RMA query filter clauses.
        
        Parameters
        ----------
        filters : dict
            keys and values for narrowing a query.
        
        Returns
        -------
        string
            filter clause for an RMA query string.
        '''
        filters_builder = []
        
        for (key, value) in filters.items():
            filters_builder.append(self.filter(key, value))
        
        return ''.join(filters_builder)
    
    # TODO: this needs to be more rigorous.
    def tuple_filters(self, filters):
        '''Construct an RMA filter clause.
        
        Notes
        -----
        
        See `RMA Path Syntax - Square Brackets for Filters <http://help.brain-map.org/display/api/RMA+Path+Syntax#RMAPathSyntax-SquareBracketsforFilters>`_ for additional documentation.
        '''
        filters_builder = []
        
        for filt in filters:
            if filt[-1] == None:
                continue
            if len(filt) == 2:
                val = filt[1]
                if type(val) is list:
                    val_array = []
                    for v in val:
                        if type(v) is str:
                            val_array.append(v)
                        else:
                            val_array.append(str(v))
                    val = ','.join(val_array)
                    filters_builder.append("[%s$eq%s]" % (filt[0], val))
                elif type(val) is int:
                    filters_builder.append("[%s$eq%d]" % (filt[0], val))
                elif type(val) is bool:
                    if val:
                        filters_builder.append("[%s$eqtrue]" % (filt[0]))
                    else:
                        filters_builder.append("[%s$eqfalse]" % (filt[0]))
                elif type(val) is str:
                    filters_builder.append("[%s$eq%s]" % (filt[0], filt[1]))
            elif len(filt) == 3:
                filters_builder.append("[%s%s%s]" % (filt[0],
                                                     filt[1],
                                                     str(filt[2])))

        
        return ''.join(filters_builder)
    
    
    def quote_string(self, the_string):
        '''Wrap a clause in single quotes.
        
        Parameters
        ----------
        the_string : string
            a clause to be included in an rma query that needs to be quoted
        
        Returns
        -------
        string
            input wrapped in single quotes
        '''
        return ''.join(["'", the_string, "'"])
    
    
    def filter(self, key, value):
        '''serialize a single RMA query filter clause.
        
        Parameters
        ----------
        key : string
            keys for narrowing a query.
        value : string
            value for narrowing a query.
        
        Returns
        -------
        string
            a single filter clause for an RMA query string.
        '''        
        return "".join(['[',
                        key,
                        RmaApi.EQ,
                        str(value),
                        ']'])
    
    
    def build_schema_query(self,  clazz=None, fmt='json'):
        '''Build the URL that will fetch the data schema.
        
        Parameters
        ----------
        clazz : string, optional
            Name of a specific class or None (default).
        fmt : string, optional
            json (default) or xml
        
        Returns
        -------
        url : string
            The constructed URL
        
        Notes
        -----
        If a class is specified, only the schema information for that class
        will be requested, otherwise the url requests the entire schema.
        '''
        if clazz != None:
            class_clause = '/' + clazz
        else:
            class_clause = ''
        
        url = ''.join([self.rma_endpoint,
                       class_clause,
                       '.',
                       fmt])
        
        return url
    
    
    def get_schema(self, clazz=None):
        '''Retrieve schema information.'''
        schema_data = self.do_query(self.build_schema_query,
                                   self.read_data,
                                   clazz)
        
        return schema_data



# Copyright 2015 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.

import allensdk.core.json_utilities as ju
import logging


class ManifestBuilder(object):
    def __init__(self):
        self._log = logging.getLogger(__name__)
        self.path_info = []
        self.bps_cfg = {}
        self.stimulus_conf = {}
        self.hoc_conf = {}
    
    
    def add_path(self, key, spec,
                 typename='dir',
                 parent_key=None,
                 format=None):
        entry = {
            'key': key,
            'type': typename,
            'spec': spec }
        
        if format != None:
            entry['format'] = format
        
        if parent_key != None:
            entry['parent_key'] = parent_key
            
        self.path_info.append(entry)
    
    
    def write_json_file(self, path):
        with open(path, 'wb') as f:
            f.write(self.write_json_string())
    
    
    def get_config(self):
        wrapper = { "manifest": self.path_info }
        wrapper.update(self.bps_cfg)
        wrapper.update(self.stimulus_conf)
        wrapper.update(self.hoc_conf)
        
        return wrapper

    
    def write_json_string(self):
        config = self.get_config()
        return ju.write_string(config)


# Copyright 2015 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.

import json, logging

from allensdk.api.queries.rma_api import RmaApi

class GlifApi(RmaApi):
    def __init__(self, base_uri=None):
        super(RmaApi, self).__init__(base_uri)

        self.neuronal_model = None
        self.ephys_sweeps = None
        self.stimulus_url = None
        self.neuron_config_url = None

    def list_neuronal_models(self):
        ''' Query the API for a list of all GLIF neuronal models. 

        Returns
        -------
        list
            Meta data for all GLIF neuronal models.
        '''

        include = "specimen(ephys_result),neuronal_model_template[name$il'*LIF*']"

        return self.model_query('NeuronalModel', include=include, num_rows='all')


    def get_neuronal_model(self, neuronal_model_id):
        '''Query the current RMA endpoint with a neuronal_model id
        to get the corresponding well known files and meta data.
        
        Returns
        -------
        dict
            A dictionary containing 
        '''
        include = ( 'neuronal_model_template(well_known_files(well_known_file_type)),' +
                    'specimen(ephys_sweeps,ephys_result(well_known_files(well_known_file_type))),' +
                    'well_known_files(well_known_file_type)' )

        criteria = "[id$eq%d]" % neuronal_model_id
 
        self.neuronal_model = self.model_query('NeuronalModel',
                                               criteria=criteria,
                                               include=include,
                                               num_rows='all')[0]

        self.ephys_sweeps = None
        self.neuron_config_url = None
        self.stimulus_url = None            

        # sweeps come from the specimen
        try:
            specimen = self.neuronal_model['specimen']
            self.ephys_sweeps = specimen['ephys_sweeps']
        except Exception, e:
            print e.message
            self.ephys_sweeps = None
            
        if self.ephys_sweeps is None:
            logging.warning("Could not find ephys_sweeps for this model (%d)" % self.neuronal_model['id'])

        
        # neuron config file comes from the neuronal model's well known files
        try:
            for wkf in self.neuronal_model['well_known_files']:
                if wkf['path'].endswith('neuron_config.json'):
                    self.neuron_config_url = wkf['download_link']
                    break
        except Exception, e:
            self.neuron_config_url = None

        if self.neuron_config_url is None:
            logging.warning("Could not find neuron config well_known_file for this model (%d)" % self.neuronal_model['id'])

        # NWB file comes from the ephys_result's well known files
        try:
            ephys_result = specimen['ephys_result']
            for wkf in ephys_result['well_known_files']:
                if wkf['well_known_file_type']['name'] == 'NWB':
                    self.stimulus_url = wkf['download_link']
                    break
        except Exception, e:
            self.stimulus_url = None

        if self.stimulus_url is None:
            logging.warning("Could not find stimulus well_known_file for this model (%d)" % self.neuronal_model['id'])


        self.metadata = {
            'neuron_config_url': self.neuron_config_url,
            'stimulus_url': self.stimulus_url,
            'ephys_sweeps': self.ephys_sweeps,
            'neuronal_model': self.neuronal_model
            }

        return self.metadata


    def get_ephys_sweeps(self):
        ''' Retrieve ephys sweep information out of downloaded metadata for a neuronal model 
        
        Returns
        -------
        list
            A list of sweeps metadata dictionaries
        '''
        return self.ephys_sweeps


    def get_neuron_config(self, output_file_name=None):
        ''' Retrieve a model configuration file from the API, optionally save it to disk, and 
        return the contents of that file as a dictionary.

        Parameters
        ----------
        output_file_name: string
            File name to store the neuron configuration (optional).
        '''
        if self.neuron_config_url is None:
            raise Exception("URL for neuron config file is empty.")

        print self.api_url + self.neuron_config_url

        neuron_config = self.retrieve_parsed_json_over_http(self.api_url + self.neuron_config_url)

        if output_file_name:
            with open(output_file_name, 'wb') as f:
                f.write(json.dumps(neuron_config, indent=2))
        
        return neuron_config


    def cache_stimulus_file(self, output_file_name):
        ''' Download the NWB file for the current neuronal model and save it to a file.

        Parameters
        ----------
        output_file_name: string
            File name to store the NWB file.
        '''
        if self.stimulus_url is None:
            raise Exception("URL for stimulus file is empty.")

        self.retrieve_file_over_http(self.api_url + self.metadata['stimulus_url'], output_file_name)




# Copyright 2015 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.

from allensdk.api.api import Api

class SynchronizationApi(Api):
    '''HTTP client for image synchronization services uses the image alignment results from 
    the Informatics Data Processing Pipeline.
    Note: all locations on SectionImages are reported in pixel coordinates 
    and all locations in 3-D ReferenceSpaces are reported in microns.
    
    See `Image to Image Synchronization <http://help.brain-map.org/display/api/Image-to-Image+Synchronization>`_
    for additional documentation.
    '''
    def __init__(self, base_uri=None):
        super(SynchronizationApi, self).__init__(base_uri)
        
    
    def get_image_to_atlas(self,
                           section_image_id,
                           x, y,
                           atlas_id):
        '''For a specified Atlas, find the closest annotated SectionImage
        and (x,y) location as defined by a seed SectionImage and seed (x,y) location.
        
        Parameters
        ----------
        section_image_id : integer
            Seed for spatial sync.
        x : float
            Pixel coordinate of the seed location in the seed SectionImage.
        y : float
            Pixel coordinate of the seed location in the seed SectionImage.
        atlas_id : int
            Target Atlas for image sync.
        
        Returns
        -------
        dict
            The parsed json response
        '''
        url = ''.join([self.image_to_atlas_endpoint,
                       '/',
                       str(section_image_id),
                       '.json',
                       '?x=%f&y=%f' % (x, y),
                       '&atlas_id=',
                       str(atlas_id)])
        
        return self.json_msg_query(url)
    
    
    def get_image_to_image(self,
                           section_image_id,
                           x, y,
                           section_data_set_ids):
        '''For a list of target SectionDataSets, find the closest SectionImage
        and (x,y) location as defined by a seed SectionImage and seed (x,y) pixel location.
        
        Parameters
        ----------
        section_image_id : integer
            Seed for spatial sync.
        x : float
            Pixel coordinate of the seed location in the seed SectionImage.
        y : float
            Pixel coordinate of the seed location in the seed SectionImage.
        section_data_set_ids : list of integers
            Target SectionDataSet IDs for image sync.
        
        Returns
        -------
        dict
            The parsed json response
        '''
        url = ''.join([self.image_to_image_endpoint,
                       '/',
                       str(section_image_id),
                       '.json',
                       '?x=%f&y=%f' % (x, y),
                       '&section_data_set_ids=',
                       ','.join(str(i) for i in section_data_set_ids)])
        
        return self.json_msg_query(url)
    
    
    def get_image_to_image_2d(self,
                              section_image_id,
                              x, y,
                              section_image_ids):
        '''For a list of target SectionImages, find the closest (x,y) location
        as defined by a seed SectionImage and seed (x,y) location.
        
        Parameters
        ----------
        section_image_id : integer
            Seed for image sync.
        x : float
            Pixel coordinate of the seed location in the seed SectionImage.
        y : float
            Pixel coordinate of the seed location in the seed SectionImage.
        section_image_ids : list of ints
            Target SectionImage IDs for image sync.
        
        Returns
        -------
        dict
            The parsed json response
        '''
        url = ''.join([self.image_to_image_2d_endpoint,
                       '/',
                       str(section_image_id),
                       '.json',
                       '?x=%f&y=%f' % (x, y),
                       '&section_image_ids=',
                       ','.join(str(i) for i in section_image_ids)])
        
        return self.json_msg_query(url)
    
        
    def get_reference_to_image(self,
                               reference_space_id,
                               x, y, z,
                               section_data_set_ids):
        '''For a list of target SectionDataSets, find the closest SectionImage
        and (x,y) location as defined by a (x,y,z) location in a specified ReferenceSpace.
        
        Parameters
        ----------
        reference_space_id : integer
            Seed for spatial sync.
        x : float
            Coordinate (in microns) of the seed location in the seed ReferenceSpace.
        y : float
            Coordinate (in microns) of the seed location in the seed ReferenceSpace.
        z : float
            Coordinate (in microns) of the seed location in the seed ReferenceSpace.
        section_data_set_ids : list of ints
            Target SectionDataSets IDs for image sync.
        
        Returns
        -------
        dict
            The parsed json response
        '''
        url = ''.join([self.reference_to_image_endpoint,
                       '/',
                       str(reference_space_id),
                       '.json',
                       '?x=%f&y=%f&z=%f' % (x, y, z),
                       '&section_data_set_ids=',
                       ','.join(str(i) for i in section_data_set_ids)])
        
        return self.json_msg_query(url)
    
    
    def get_image_to_reference(self,
                               section_image_id,
                               x, y):
        '''For a specified SectionImage and (x,y) location,
        return the (x,y,z) location in the ReferenceSpace of the associated SectionDataSet.
        
        Parameters
        ----------
        section_image_id : integer
            Seed for image sync.
        x : float
            Pixel coordinate on the specified SectionImage.
        y : float
            Pixel coordinate on the specified SectionImage.
        
        Returns
        -------
        dict
            The parsed json response
        '''
        url = ''.join([self.image_to_reference_endpoint,
                       '/',
                       str(section_image_id),
                       '.json',
                       '?x=%f&y=%f' % (x, y)])
        
        return self.json_msg_query(url)
    
    
    def get_structure_to_image(self, 
                               section_data_set_id,
                               structure_ids):
        '''For a list of target structures, find the closest SectionImage 
        and (x,y) location as defined by the centroid of each Structure.
        
        Parameters
        ----------
        section_data_set_id : integer
            primary key
        structure_ids : list of integers
            primary key
        
        Returns
        -------
        dict
            The parsed json response
        '''
        url = ''.join([self.structure_to_image_endpoint,
                       '/',
                       str(section_data_set_id),
                       '.json',
                       '?structure_ids=',
                       ','.join([str(i) for i in structure_ids])])
        
        return self.json_msg_query(url)


import unittest
from mock import MagicMock
from allensdk.api.queries.rma_api import RmaApi
import pandas as pd
import pandas.io.json as pj
import allensdk.core.json_utilities as ju
from allensdk.api.queries.biophysical_perisomatic_api import \
    BiophysicalPerisomaticApi
from allensdk.config.manifest import Manifest
from allensdk.api.queries.mouse_connectivity_api import MouseConnectivityApi
from allensdk.model.biophysical_perisomatic.runner import run
import os
import subprocess
from allensdk.model.biophys_sim.config import Config
from allensdk.core.nwb_data_set import NwbDataSet
import numpy


class BiophysicalTests(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(BiophysicalTests, self).__init__(*args, **kwargs)
    
    
    def setUp(self):
        self.mca = MouseConnectivityApi()
    
    
    def tearDown(self):
        self.cache = None
        self.manifest = None
        self.mca = None
    
    @unittest.skip("requires a big NWB file, takes a while.")
    def test_spike_times(self):
        expected = [
            2.937305,   3.16453 ,   3.24271 ,   4.1622  ,   4.24182 ,
            10.0898  ,  10.132545,  10.176095,  10.2361  ,  10.660655,
            10.762125,  10.863465,  10.93833 ,  11.140815,  11.19246 ,
            11.24553 ,  11.696305,  11.812655,  11.90469 ,  12.056755,
            12.15794 ,  12.233905,  12.47577 ,  12.741295,  12.82861 ,
            12.923175,  18.05068 ,  18.139875,  18.17693 ,  18.221485,
            18.24337 ,  18.39981 ,  18.470705,  18.759675,  18.82183 ,
            18.877475,  18.91033 ,  18.941195,  19.050515,  19.12557 ,
            19.15963 ,  19.188655,  19.226205,  19.29813 ,  19.420665,
            19.47627 ,  19.763365,  19.824225,  19.897995,  19.93155 ,
            20.04916 ,  20.11832 ,  20.148755,  20.18004 ,  20.22173 ,
            20.2433  ,  20.40018 ,  20.470915,  20.759715,  20.82156 ,
            20.866465,  20.90807 ,  20.939175]
        
        bp = BiophysicalPerisomaticApi('http://api.brain-map.org')
        bp.cache_stimulus = True # change to False to not download the large stimulus NWB file
        neuronal_model_id = 472451419    # get this from the web site as above
        bp.cache_data(neuronal_model_id, working_directory='neuronal_model')
        cwd = os.path.realpath(os.curdir)
        print(cwd)
        os.chdir(os.path.join(cwd, 'neuronal_model'))
        manifest = ju.read('manifest.json')
        manifest['biophys'][0]['model_file'][0] = 'manifest_51.json'
        manifest['runs'][0]['sweeps'] = [51]
        ju.write('manifest_51.json', manifest)
        subprocess.call(['nrnivmodl', './modfiles'])
        run(Config().load('manifest_51.json'))
        #os.chdir(cwd)
        nwb_out = NwbDataSet('work/386049444.nwb')
        spikes = nwb_out.get_spike_times(51)
        
        numpy.testing.assert_array_almost_equal(spikes, expected)



# Copyright 2015 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.

import unittest
from mock import patch, mock_open
from allensdk.config.model.description_parser import DescriptionParser

class PyconfigParserTests(unittest.TestCase):
    def setUp(self):
        self.description = None
        
        file_1 = ("{\n"
                  "    \"section_A\": [\n"
                  "       {\n"
                  "           \"prop_a\": \"val_a\",\n"
                  "           \"prop_b\": \"val_b\"\n"
                  "       },\n"
                  "       {\n"
                  "           \"prop_c\": \"val_c\",\n"
                  "           \"prop_d\": \"val_d\"\n"
                  "       }\n"
                  "    ],\n"
                  "    \"section_B\": [\n"
                  "       {\n"
                  "           \"prop_e\": \"val_e\",\n"
                  "           \"prop_f\": \"val_f\"\n"
                  "       },\n"
                  "       {\n"
                  "           \"prop_g\": \"val_g\",\n"
                  "           \"prop_h\": \"val_h\"\n"
                  "       }\n"
                  "    ]\n"
                  "}\n"
                  )
        file_2 = ("{\n"
                  "    \"section_B\": [\n"
                  "       {\n"
                  "           \"prop_i\": \"val_i\",\n"
                  "           \"prop_j\": \"val_j\"\n"
                  "       }\n"
                  "    ],\n"
                  "    \"section_C\": [\n"
                  "       {\n"
                  "           \"prop_k\": \"val_k\",\n"
                  "           \"prop_l\": \"val_l\"\n"
                  "       }\n"
                  "    ]\n"
                  "}\n"
                  )
        
        with patch("__builtin__.open",
                   mock_open(
                       read_data=file_1)):
            parser = DescriptionParser()
            self.description = parser.read("mock_1.pycfg")
        
        with patch("__builtin__.open",
                   mock_open(
                       read_data=file_2)):
            parser = DescriptionParser()
            parser.read("mock_2.pycfg",
                        self.description)
    
    
    def testAllSectionsPresent(self):
        self.assertTrue('section_A' in self.description.data and
                        'section_B' in self.description.data and
                        'section_C' in self.description.data)
        self.assertTrue(len(self.description.data.keys()) == 3)
    
    
    def testSectionA(self):
        self.assertEqual(len(self.description.data['section_A']),
                         2)
        self.assertDictEqual(
            self.description.data['section_A'][0],
            { 'prop_a': 'val_a',
              'prop_b': 'val_b' })
        self.assertDictEqual(
            self.description.data['section_A'][1],
            { 'prop_c': 'val_c',
              'prop_d': 'val_d' })
    
    
    def testSectionB(self):
        self.assertEqual(len(self.description.data['section_B']),
                         3)
        self.assertDictEqual(
            self.description.data['section_B'][0],
            { 'prop_e': 'val_e',
              'prop_f': 'val_f' })
        self.assertDictEqual(
            self.description.data['section_B'][1],
            { 'prop_g': 'val_g',
              'prop_h': 'val_h' })
        self.assertDictEqual(
            self.description.data['section_B'][2],
            { 'prop_i': 'val_i',
              'prop_j': 'val_j' })
    
    
    def testSectionC(self):
        self.assertEqual(len(self.description.data['section_C']),
                         1)
        self.assertDictEqual(
            self.description.data['section_C'][0],
            { 'prop_k': 'val_k',
              'prop_l': 'val_l' })



from allensdk.api.queries.annotated_section_data_sets_api import \
    AnnotatedSectionDataSetsApi

import unittest, json
from mock import MagicMock

class AnnotatedSectionDataSetsApiTests(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(AnnotatedSectionDataSetsApiTests, self).__init__(*args, **kwargs)
    
    
    def setUp(self):
        self.asdsa = AnnotatedSectionDataSetsApi()
    
    
    def tearDown(self):
        self.asdsa = None


    def test_get_annotated_section_data_set(self):
        expected = "http://api.brain-map.org/api/v2/annotated_section_data_sets.json?structures=112763676&intensity_values='High','Low','Medium'&density_values='High','Low'&pattern_values='Full'&age_names='E11.5','13.5'"
        
        self.asdsa.json_msg_query = \
            MagicMock(name='json_msg_query')
               
        self.asdsa.get_annotated_section_data_sets(
            structures=[112763676],
            intensity_values=["High", "Low", "Medium"],
            density_values=["High","Low"],
            pattern_values=["Full"],
            age_names=["E11.5", "13.5"])
        
        self.asdsa.json_msg_query.assert_called_once_with(expected)


    def test_get_compound_annotated_section_data_set(self):
        expected = "http://api.brain-map.org/api/v2/annotated_section_data_sets.json?structures=112763676&intensity_values='High','Low','Medium'&density_values='High','Low'&pattern_values='Full'&age_names='E11.5','13.5'"
        
        self.asdsa.json_msg_query = \
            MagicMock(name='json_msg_query')
               
        self.asdsa.get_annotated_section_data_sets(
            structures=[112763676],
            intensity_values=["High", "Low", "Medium"],
            density_values=["High","Low"],
            pattern_values=["Full"],
            age_names=["E11.5", "13.5"])
        
        self.asdsa.json_msg_query.assert_called_once_with(expected)
    

    def test_get_annotated_section_data_set_via_rma(self):
        expected = "http://api.brain-map.org/api/v2/compound_annotated_section_data_sets.json?query=[structures $in 112763676 : intensity_values $in 'High','Low'] or [structures $in 112763686 : intensity_values $in 'Low']"
        
        self.asdsa.json_msg_query = \
            MagicMock(name='json_msg_query')
               
        self.asdsa.get_compound_annotated_section_data_sets(
            [{'structures' : [112763676],
              'intensity_values' : ['High', 'Low'],
              'link' : 'or' },
             {'structures' : [112763686],
              'intensity_values' : ['Low']}])
        
        self.asdsa.json_msg_query.assert_called_once_with(expected)


import unittest
from mock import MagicMock
from allensdk.api.cache import Cache
from allensdk.api.queries.rma_api import RmaApi
import allensdk.core.json_utilities as ju
import pandas as pd
import pandas.io.json as pj


class CacheTests(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(CacheTests, self).__init__(*args, **kwargs)
    
    
    def setUp(self):
        self.cache = Cache()
        self.api = RmaApi()
    
    
    def tearDown(self):
        self.cache = None
        self.api = None
    
    
    def test_wrap_json(self):
        msg = [ { 'whatever': True } ]
        
        ju.read_url_get = \
            MagicMock(name='read_url_get',
                      return_value={ 'msg': msg })
        ju.write = \
            MagicMock(name='write')
#        pj.read_json = \
#            MagicMock(name='read_json',
#                      return_value=msg)
        ju.read = \
            MagicMock(name='read',
                      return_value=pd.DataFrame(msg))
               
        df = self.cache.wrap(self.api.model_query,
                             'example.txt',
                             cache=True,
                             model='Hemisphere')
        
        self.assertTrue(df.loc[:,'whatever'][0])
        
        ju.read_url_get.assert_called_once_with('http://api.brain-map.org/api/v2/data/query.json?q=model::Hemisphere')
        ju.write.assert_called_once_with('example.txt', msg)
        ju.read.assert_called_once_with('example.txt')
#        pj.read_json.assert_called_once_with('example.txt')
    
    
    def test_wrap_dataframe(self):
        msg = [ { 'whatever': True } ]
        
        ju.read_url_get = \
            MagicMock(name='read_url_get',
                      return_value={ 'msg': msg })
        ju.write = \
            MagicMock(name='write')
        pj.read_json = \
            MagicMock(name='read_json',
                      return_value=msg)
        
        json_data = self.cache.wrap(self.api.model_query,
                                    'example.txt',
                                    cache=True,
                                    return_dataframe=True,
                                    model='Hemisphere')
        
        self.assertTrue(json_data[0]['whatever'])
        
        ju.read_url_get.assert_called_once_with('http://api.brain-map.org/api/v2/data/query.json?q=model::Hemisphere')
        ju.write.assert_called_once_with('example.txt', msg)
        pj.read_json.assert_called_once_with('example.txt', orient='records')


# Copyright 2015 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# Merchantability Or Fitness FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.

import numpy as np
import json
import re
import urllib2, urlparse

def read(file_name):
    """ Shortcut reading JSON from a file. """
    with open(file_name, 'rb') as f:
        return json.loads(f.read())


def write(file_name, obj):
    """ Shortcut for writing JSON to a file.  This also takes care of serializing numpy and data types. """
    with open(file_name, 'wb') as f:
        f.write(write_string(obj))


def write_string(obj):
    """ Shortcut for writing JSON to a string.  This also takes care of serializing numpy and data types. """
    return json.dumps(obj, indent=2, default=json_handler)


def read_url(url, method='POST'):
    if method == 'GET':
        return read_url_get(url)
    elif method == 'POST':
        return read_url_post(url)
    else:
        raise Exception('Unknown request method: (%s)' % method)


def read_url_get(url):
    '''Transform a JSON contained in a file into an equivalent
    nested python dict.
    
    Parameters
    ----------
    url : string
    where to get the json.
    
    Returns
    -------
    dict
    Python version of the input
    
    Note: if the input is a bare array or literal, for example,
    the output will be of the corresponding type.
    '''
    response = urllib2.urlopen(url)
    json_string = response.read()
    
    return json.loads(json_string)


def read_url_post(url):
    '''Transform a JSON contained in a file into an equivalent
    nested python dict.
    
    Parameters
    ----------
    url : string
    where to get the json.
    
    Returns
    -------
    dict
    Python version of the input
    
    Note: if the input is a bare array or literal, for example,
    the output will be of the corresponding type.
    '''
    urlp = urlparse.urlparse(url)
    main_url = urlparse.urlunsplit((urlp.scheme, urlp.netloc, urlp.path, '', ''))
    data = json.dumps(dict(urlparse.parse_qsl(urlp.query)))

    handler = urllib2.HTTPHandler()
    opener = urllib2.build_opener(handler)

    request = urllib2.Request(main_url, data)
    request.add_header("Content-Type",'application/json')
    request.get_method = lambda: 'POST'
    
    try:
        response = opener.open(request)
    except Exception, e:
        response = e
        
    if response.code == 200:
        json_string = response.read()
    else:
        json_string = response.read()
        
    return json.loads(json_string)


def json_handler(obj):
    """ Used by write_json convert a few non-standard types to things that the json package can handle. """
    if hasattr(obj, 'to_dict'):
        return obj.to_dict()
    elif isinstance(obj, np.ndarray):
        return obj.tolist()
    elif ( isinstance(obj, np.float64) or 
           isinstance(obj, np.float32) or
           isinstance(obj, np.float16) ):
        return float(obj)
    elif ( isinstance(obj, np.int64) or 
           isinstance(obj, np.int32) or 
           isinstance(obj, np.int16) or 
           isinstance(obj, np.int8) or 
           isinstance(obj, np.uint64) or 
           isinstance(obj, np.uint32) or
           isinstance(obj, np.uint16) or
           isinstance(obj, np.uint8) ):
        return long(obj)
    elif hasattr(obj, 'isoformat'):
        return obj.isoformat()
    else:
        raise TypeError, 'Object of type %s with value of %s is not JSON serializable' % (type(obj), repr(obj))


class JsonComments(object):
    _oneline_comment = re.compile(r"\/\/.*$",
                                  re.MULTILINE)
    _multiline_comment_start = re.compile(r"\/\*",
                                          re.MULTILINE |
                                          re.DOTALL)
    _multiline_comment_end = re.compile(r"\*\/",
                                        re.MULTILINE |
                                        re.DOTALL)
    _blank_line = re.compile(r"\n?^\s*$", re.MULTILINE)
    _carriage_return = re.compile(r"\r$", re.MULTILINE)
    

    @classmethod 
    def read_string(cls, json_string):
        json_string_no_comments = cls.remove_comments(json_string)
        return json.loads(json_string_no_comments)        


    @classmethod
    def read_file(cls, file_name):
        with open(file_name) as f:
            json_string = f.read()
            return cls.read_string(json_string)


    @classmethod
    def remove_comments(cls, json_string):
        '''Strip single and multiline javascript-style comments.
        
        Parameters
        ----------
        json : string
            Json string with javascript-style comments.
            
        Returns
        -------
        string
            Copy of the input with comments removed.
            
        Note: A JSON decoder MAY accept and ignore comments.
        '''
        json_string = JsonComments._oneline_comment.sub('', json_string)
        json_string = JsonComments._carriage_return.sub('', json_string)
        json_string = JsonComments.remove_multiline_comments(json_string)
        json_string = JsonComments._blank_line.sub('', json_string)
        
        return json_string
    
    
    @classmethod
    def remove_multiline_comments(cls, json_string):
        '''Rebuild input without substrings matching /*...*/.
        
        Parameters
        ----------
        json_string : string
            may or may not contain multiline comments.
        
        Returns
        -------
        string
            Copy of the input without the comments.
        '''
        new_json = []
        start_iter = JsonComments._multiline_comment_start.finditer(json_string)
        json_slice_start = 0
        
        for comment_start in start_iter:
            json_slice_end = comment_start.start()
            new_json.append(json_string[json_slice_start:json_slice_end])
            search_start = comment_start.end()
            comment_end = JsonComments._multiline_comment_end.search(json_string[search_start:])
            if comment_end == None:
                break
            else:
                json_slice_start = search_start + comment_end.end()
        new_json.append(json_string[json_slice_start:])
        
        return ''.join(new_json)



# Copyright 2015 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.

import numpy

class DatUtilities(object):
    
    @classmethod
    def save_voltage(cls, output_path, v, t):
        '''Save a single voltage output result into a simple text format.
        
        The output file is one t v pair per line.
        
        Parameters
        ----------
        output_path : string
            file name for output
        v : numpy array
            voltage
        t : numpy array
            time
        '''
        data = numpy.transpose(numpy.vstack((t, v)))
        with open (output_path, "w") as f:
            numpy.savetxt(f, data)


# Copyright 2015 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.

import urllib2
import allensdk.core.json_utilities as json_utilities
import pandas as pd
import logging


class Api(object):
    _log = logging.getLogger(__name__)
    default_api_url = 'http://api.brain-map.org'
    download_url = 'http://download.alleninstitute.org'
    
    def __init__(self, api_base_url_string=None):
        if api_base_url_string==None:
            api_base_url_string=Api.default_api_url
            
        self.set_api_urls(api_base_url_string)
        self.default_working_directory = None
    
    
    def set_api_urls(self, api_base_url_string):
        '''Set the internal RMA and well known file download endpoint urls
        based on a api server endpoint.
        
        Parameters
        ----------
        api_base_url_string : string
            url of the api to point to
        '''
        self.api_url = api_base_url_string
        
        # http://help.brain-map.org/display/api/Downloading+a+WellKnownFile
        self.well_known_file_endpoint = api_base_url_string + '/api/v2/well_known_file_download'
        
        # http://help.brain-map.org/display/api/Downloading+3-D+Expression+Grid+Data
        self.grid_data_endpoint = api_base_url_string + '/grid_data'
        
        # http://help.brain-map.org/display/api/Downloading+and+Displaying+SVG
        self.svg_endpoint = api_base_url_string + '/api/v2/svg'
        self.svg_download_endpoint = api_base_url_string + '/api/v2/svg_download'
        
        # http://help.brain-map.org/display/api/Downloading+an+Ontology%27s+Structure+Graph
        self.structure_graph_endpoint = api_base_url_string + '/api/v2/structure_graph_download'
        
        # http://help.brain-map.org/display/api/Searching+a+Specimen+or+Structure+Tree
        self.tree_search_endpoint = api_base_url_string + '/api/v2/tree_search'
        
        # http://help.brain-map.org/display/api/Searching+Annotated+SectionDataSets
        self.annotated_section_data_sets_endpoint = api_base_url_string + '/api/v2/annotated_section_data_sets'
        self.compound_annotated_section_data_sets_endpoint = api_base_url_string + '/api/v2/compound_annotated_section_data_sets'
        
        # http://help.brain-map.org/display/api/Image-to-Image+Synchronization#Image-to-ImageSynchronization-ImagetoImage
        self.image_to_atlas_endpoint = api_base_url_string + '/api/v2/image_to_atlas'
        self.image_to_image_endpoint = api_base_url_string + '/api/v2/image_to_image'
        self.image_to_image_2d_endpoint = api_base_url_string + '/api/v2/image_to_image_2d'
        self.reference_to_image_endpoint = api_base_url_string + '/api/v2/reference_to_image'
        self.image_to_reference_endpoint = api_base_url_string + '/api/v2/image_to_reference'
        self.structure_to_image_endpoint = api_base_url_string + '/api/v2/structure_to_image'
        
        # http://help.brain-map.org/display/mouseconnectivity/API
        self.section_image_download_endpoint = api_base_url_string + '/api/v2/section_image_download'
        self.atlas_image_download_endpoint = api_base_url_string + '/api/v2/atlas_image_download'
        self.projection_image_download_endpoint = api_base_url_string + '/api/v2/projection_image_download'
        self.informatics_archive_endpoint = Api.download_url + '/informatics-archive'
        
        self.rma_endpoint = api_base_url_string + '/api/v2/data'
    
    
    def set_default_working_directory(self, working_directory):
        '''Set the working directory where files will be saved.
        
        Parameters
        ----------
        working_directory : string
             the absolute path string of the working directory.
        '''
        self.default_working_directory = working_directory
    
    
    def read_data(self, parsed_json):
        '''Return the message data from the parsed query.
        
        Parameters
        ----------
        parsed_json : dict
            A python structure corresponding to the JSON data returned from the API.
        
        Notes
        -----
        See `API Response Formats - Response Envelope <http://help.brain-map.org/display/api/API+Response+Formats#APIResponseFormats-ResponseEnvelope>`_
        for additional documentation.
        '''
        return parsed_json['msg']
    
    
    def json_msg_query(self, url, dataframe=False):
        ''' Common case where the url is fully constructed
            and the response data is stored in the 'msg' field.
        
        Parameters
        ----------
        url : string
            Where to get the data in json form
        dataframe : boolean
            True converts to a pandas dataframe, False (default) doesn't
        
        Returns
        -------
        dict or DataFrame
            returned data; type depends on dataframe option
        '''
        
        data = self.do_query(lambda *a, **k:  url,
                             self.read_data)
        
        if dataframe == True:
            data = pd.DataFrame(data)
        
        return data
    
    
    def do_query(self, url_builder_fn, json_traversal_fn, *args, **kwargs):
        '''Bundle an query url construction function
        with a corresponding response json traversal function.
        
        Parameters
        ----------
        url_builder_fn : function
            A function that takes parameters and returns an rma url.
        json_traversal_fn : function
            A function that takes a json-parsed python data structure and returns data from it.
        post : boolean, optional kwarg
            True does an HTTP POST, False (default) does a GET
        args : arguments
            Arguments to be passed to the url builder function.
        kwargs : keyword arguments
            Keyword arguments to be passed to the rma builder function.
        
        Returns
        -------
        any type
            The data extracted from the json response.
        
        Examples
        --------
        `A simple Api subclass example
        <data_api_client.html#creating-new-api-query-classes>`_.
        '''
        api_url = url_builder_fn(*args, **kwargs)
        
        post = kwargs.get('post', False)
        
        json_parsed_data = self.retrieve_parsed_json_over_http(api_url, post)
        
        return json_traversal_fn(json_parsed_data)
    
    
    def do_rma_query(self, rma_builder_fn, json_traversal_fn, *args, **kwargs):
        '''Bundle an RMA query url construction function
        with a corresponding response json traversal function.
        
        ..note:: Deprecated in AllenSDK 0.9.2
            `do_rma_query` will be removed in AllenSDK 1.0, it is replaced by
            `do_query` because the latter is more general.
        
        Parameters
        ----------
        rma_builder_fn : function
            A function that takes parameters and returns an rma url.
        json_traversal_fn : function
            A function that takes a json-parsed python data structure and returns data from it.
        args : arguments
            Arguments to be passed to the rma builder function.
        kwargs : keyword arguments
            Keyword arguments to be passed to the rma builder function.
        
        Returns
        -------
        any type
            The data extracted from the json response.
        
        Examples
        --------
        `A simple Api subclass example
        <data_api_client.html#creating-new-api-query-classes>`_.
        '''
        return self.do_query(rma_builder_fn, json_traversal_fn, *args, **kwargs)
    
    
    def load_api_schema(self):
        '''Download the RMA schema from the current RMA endpoint
        
        Returns
        -------
        dict
            the parsed json schema message
        
        Notes
        -----
        This information and other 
        `Allen Brain Atlas Data Portal Data Model <http://help.brain-map.org/display/api/Data+Model>`_
        documentation is also available as a
        `Class Hierarchy <http://api.brain-map.org/class_hierarchy>`_
        and `Class List <http://api.brain-map.org/class_hierarchy>`_.
        
        '''
        schema_url = self.rma_endpoint + '/enumerate.json'
        json_parsed_schema_data = self.retrieve_parsed_json_over_http(schema_url)
        
        return json_parsed_schema_data
    
    
    def construct_well_known_file_download_url(self, well_known_file_id):
        '''Join data api endpoint and id.
        
        Parameters
        ----------
        well_known_file_id : integer or string representing an integer
            well known file id
        
        Returns
        -------
        string
            the well-known-file download url for the current api api server
        
        See Also
        --------
        retrieve_file_over_http: Can be used to retrieve the file from the url.
        '''
        return self.well_known_file_endpoint + '/' + str(well_known_file_id)
    
    
    def retrieve_file_over_http(self, url, file_path):
        '''Get a file from the data api and save it.
        
        Parameters
        ----------
        url : string
            Url[1]_ from which to get the file.
        file_path : string
            Absolute path including the file name to save.
        
        See Also
        --------
        construct_well_known_file_download_url: Can be used to construct the url.
        
        References
        ----------
        .. [1] Allen Brain Atlas Data Portal: `Downloading a WellKnownFile <http://help.brain-map.org/display/api/Downloading+a+WellKnownFile>`_.
        '''
        try:
            with open(file_path, 'wb') as f:
                response = urllib2.urlopen(url)
                f.write(response.read())
        except urllib2.HTTPError:
            self._log.error("Couldn't retrieve file from %s" % url)
            raise
    
    
    def retrieve_parsed_json_over_http(self, url, post=False):
        '''Get the document and put it in a Python data structure
        
        Parameters
        ----------
        url : string
            Full API query url.
        post : boolean
            True does an HTTP POST, False (default) encodes the URL and does a GET
        
        Returns
        -------
        dict
            Result document as parsed by the JSON library.
        '''
        
        if post == False:
            data = json_utilities.read_url(urllib2.quote(url,';/?:@&=+$,'),
                                           'GET')
        else:
            data = json_utilities.read_url(url, 'POST')
        
        return data
    
    
    def retrieve_xml_over_http(self, url):
        '''Get the document and put it in a Python data structure
        
        Parameters
        ----------
        url : string
            Full API query url.
        
        Returns
        -------
        string
            Unparsed xml string.
        '''
        response = urllib2.urlopen(url)
        
        return response.read()




import unittest
from mock import MagicMock
from allensdk.api.api import Api
import allensdk.core.json_utilities as ju

class ApiTests(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(ApiTests, self).__init__(*args, **kwargs)
    
    
    def setUp(self):
        self.api = Api()
    
    
    def tearDown(self):
        self.api = None
    
    
    def test_do_query_post(self):
        ju.read_url_post = \
            MagicMock(name='read_url_post',
                      return_value={ 'whatever': True })
               
        self.api.do_query(lambda *a, **k: 'http://localhost/%s' % (a[0]),
                          lambda d: d,
                          "wow",
                          post=True)
        
        ju.read_url_post.assert_called_once_with('http://localhost/wow')
    
    
    def test_do_query_get(self):
        ju.read_url_get = \
            MagicMock(name='read_url_get',
                      return_value={ 'whatever': True })
               
        self.api.do_query(lambda *a, **k: 'http://localhost/%s' % (a[0]),
                          lambda d: d,
                          "wow",
                          post=False)
        
        ju.read_url_get.assert_called_once_with('http://localhost/wow')
    
    
    def test_load_api_schema(self):
        ju.read_url_get = \
            MagicMock(name='read_url_get',
                      return_value={ 'whatever': True })
               
        self.api.load_api_schema()
        
        ju.read_url_get.assert_called_once_with('http://api.brain-map.org/api/v2/data/enumerate.json')



# Copyright 2015 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.
from allensdk.api.queries.rma_api import RmaApi

class ConnectedServices(object):
    '''
    A class representing a schema of informatics web services.
    
    Notes
    -----
    See `Connected Services and Pipes <http://help.brain-map.org/display/api/Connected+Services+and+Pipes>`_
    for a human-readable list of services and parameters.
    
    The URL format is documented at
    `Service Pipelines <http://help.brain-map.org/display/api/Service+Pipelines>`_.
    
    Connected Services only include API services that are accessed
    via the RMA endpoint using an rma::services stage. 
    '''
    ARRAY = 'array'
    STRING = 'string'
    INTEGER = 'integer'
    FLOAT = 'float'
    BOOLEAN = 'boolean'
    
    def __init__(self):
        pass
    
    
    def build_url(self, service_name, kwargs):
        '''Create a single stage RMA url from a service name and parameters.
        '''
        rma = RmaApi()
        fmt = kwargs.get('fmt', 'json')
        
        schema_entry = ConnectedServices._schema[service_name]
        
        params = []
        
        for parameter in schema_entry['parameters']:
            value = kwargs.get(parameter['name'], None)
            if value != None:
                params.append((parameter['name'], value))
        
        service_stage = rma.service_stage(service_name,
                                          params)
        
        url = rma.build_query_url([service_stage], fmt)
        
        return url
    
    @classmethod
    @property
    def schema(cls):
        '''Dictionary of service names and parameters.
        
        Notes
        -----
        See `Connected Services and Pipes <http://help.brain-map.org/display/api/Connected+Services+and+Pipes>`_
        for a human-readable list of connected services and their parameters.
        '''
        return cls._schema
    
    _schema = {
        'dev_human_correlation' : {
            'parameters' : [
                { 'name' : 'set',
                  'optional' : True,
                  'type' : STRING,
                  'values' : ['rna_seq_genes',
                              'rna_seq_exons',
                              'exon_microarray_genes'
                              'exon_microarray_exons']
                },
                { 'name' : 'donors',
                  'optional' : True,
                  'type' : ARRAY
                },
                { 'name' : 'structures',
                  'optional' : False,
                  'type' : ARRAY
                },
                { 'name' : 'probes',
                  'optional' : False,
                  'type' : INTEGER
                },
                { 'name' : 'sort_order',
                  'type' : STRING,
                  'optional' : True,
                  'values' : ['asc', 'desc'],
                  'default' : ['desc']
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000 
                }
            ]
        },
        'dev_human_differential' : {
            'parameters' : [
                { 'name' : 'set',
                  'type' : STRING,
                  'values' : [ 'rna_seq_genes',
                               'rna_seq_exons',
                               'exon_microarray_genes',
                               'exon_microarray_exons' ]
                },
                { 'name' : 'donors1',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'structures1',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'donors2',
                  'type' : ARRAY,
                  'optional' : True,
                  'array_type' : INTEGER
                },
                { 'name' : 'structures2',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'sort_by',
                  'type' : STRING,
                  'optional' : True,
                  'values' : [ 'p-value', 'fold-change' ],
                  'default' : 'p-value'
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'dev_human_expression' : {
            'parameters' : [
                { 'name' : 'set',
                  'type' : STRING,
                  'values' : [ 'rna_seq_genes',
                               'rna_seq_exons',
                               'exon_microarray_genes',
                               'exon_microarray_exons' ]
                },
                { 'name' : 'probes',
                  'type' : INTEGER
                },
                { 'name' : 'donors',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'structures',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'dev_human_microarray_correlation' : {
            'parameters' : [
                { 'name' : 'donors',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'structures',
                  'type' : ARRAY,
                  'array_type' : [ INTEGER, STRING ]
                },
                { 'name' : 'probes',
                  'type' : INTEGER
                },
                { 'name' : 'sort_order',
                  'type' : STRING,
                  'optional' : True,
                  'values' : [ 'asc', 'desc' ],
                  'default' : 'desc'
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'dev_human_microarray_differential' : {
            'parameters' : [
                { 'name' : 'donors1',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'structures1',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'donors2',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'structures2',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'sort_by',
                  'type' : STRING,
                  'optional' : True,
                  'values' : [ 'p-value', 'fold-change' ],
                  'default' : 'p-value'
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'dev_human_microarray_expression' : {
            'parameters' : [
                { 'name' : 'probes',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                 },
                 { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                },
                { 'name' : 'donors',
                  'type' : INTEGER,
                  'optional' : True,
                },
                { 'name' : 'structures', 
                  'type' : INTEGER,
                  'optional' : True
                }
            ]
        },
        'dev_mouse_agea' : {
            'parameters' : [
                { 'name' : 'seed_age',
                  'type' : STRING
                },
                { 'name' : 'map_age',
                  'type' : STRING
                },
                { 'name' : 'seed_point',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'seed_threshold',
                  'type' : ARRAY,
                  'array_type' : FLOAT
                },
                { 'name' : 'map_threshold',
                  'type' : ARRAY,
                  'array_type' : FLOAT
                },
                { 'name' : 'contrast_threshold',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'target_threshold',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'dev_mouse_correlation' : {
            'parameters' : [
                { 'name' : 'row',
                  'type' : INTEGER
                },
                { 'name' : 'structures',
                  'type' : ARRAY,
                  'array_type' : [ INTEGER, STRING ],
                  'optional' : True
                },
                { 'name' : 'ages',
                  'type' : ARRAY,
                  'array_type' : STRING,
                  'optional' : True
                },
                { 'name' : 'sort_order',
                  'type' : STRING,
                  'optional' : True,
                  'values' : [ 'asc', 'desc' ],
                  'default' : 'desc'
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'gbm_correlation' : {
            'parameters' : [
                { 'name' : 'donors',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'structures',
                  'type' : ARRAY,
                  'array_type' : [ INTEGER, STRING ]
                },
                { 'name' : 'probes',
                  'type' : INTEGER
                },
                { 'name' : 'sort_order',
                  'type' : STRING,
                  'optional' : True,
                  'values' : [ 'asc', 'desc' ],
                  'default' : 'desc'
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'gbm_differential' : {
            'parameters' : [
                { 'name' : 'donors1',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'structures1',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'donors2',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'structures2',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'sort_by',
                  'type' : STRING,
                  'optional' : True,
                  'values' : [ 'p-value', 'fold-change' ],
                  'default' : 'p-value'
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'gbm_expression' : {
            'parameters' : [
                { 'name' : 'probes',
                  'type' : INTEGER,
                  'array_type' : INTEGER
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                },
                { 'name' : 'donors', 
                  'type' : INTEGER,
                  'optional': True
                }, 
                { 'name' : 'structures',
                  'type' : INTEGER,
                  'optional' : True
                } 
            ]
         },
        'gbm_ish_differential' : {
            'parameters' : [
                { 'name' : 'structures1',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'structures2',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'threshold1',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'threshold2',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
         },
        'gbm_ish_expression' : {
            'parameters' : [
                { 'name' : 'structures',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'threshold',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'human_microarray_correlation' : {
            'parameters' : [
                { 'name' : 'donors',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'structures',
                  'type' : ARRAY,
                  'array_type': [ INTEGER, STRING ]
                },
                { 'name' : 'probes',
                  'type' : INTEGER
                },
                { 'name' : 'sort_order',
                  'type' : STRING,
                  'optional' : True,
                  'values' : [ 'asc', 'desc' ],
                  'default' : 'desc'
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'human_microarray_differential' : {
            'parameters' : [
                { 'name' : 'donors1',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'structures1',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'donors2',
                  'type' : ARRAY,
                  'optional' : True,
                  'array_type' : INTEGER
                },
                { 'name' : 'structures2',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'sort_by',
                  'type' : STRING,
                  'values' : [ 'p-value', 'fold-change' ],
                  'default' : 'p-value'
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'human_microarray_expression' : {
            'parameters' : [
                { 'name' : 'probes',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'donors', 
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'structures',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
         },
        'mouse_agea' : {
            'parameters' : [
                { 'name' : 'set',
                  'type' : STRING
                },
                { 'name' : 'seed_age',
                  'type' : STRING
                },
                { 'name' : 'map_age',
                  'type' : STRING
                },
                { 'name' : 'seed_point',
                  'type' : ARRAY,
                  'array_type' : FLOAT
                },
                { 'name' : 'correlation_threshold1',
                  'type' : FLOAT,
                  'optional' : True,
                },
                { 'name' : 'correlation_threshold2',
                  'type' : FLOAT,
                  'optional' : True
                },
                { 'name' : 'threshold1',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'threshold2',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'mouse_correlation' : {
            'parameters' : [
                { 'name' : 'set',
                  'type' : STRING,
                  'values' : [ 'mouse', 'mouse_coronal' ]
                },
                { 'name' : 'structures',
                  'type' : ARRAY,
                  'array_type' : [ INTEGER, STRING ],
                  'optional' : True
                },
                { 'name' : 'row',
                  'type' : INTEGER
                },
                { 'name' : 'sort_order',
                  'type' : STRING,
                  'optional' : True,
                  'values' : [ 'asc', 'desc' ],
                  'default' : 'desc'
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'mouse_differential' : {
            'parameters' : [
                { 'name' : 'set',
                  'type' : STRING,
                  'values' : [ 'mouse', 'mouse_coronal' ]
                },
                { 'name' : 'structures1',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                }, 
                { 'name' : 'structures2',
                  'type' : ARRAY,
                  'array_type' : INTEGER 
                },
                { 'name' : 'threshold1',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'threshold2',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'mouse_connectivity_correlation' : {
            'parameters' : [
                { 'name' : 'row',
                  'type' : INTEGER
                },
                { 'name' : 'structures',
                  'type' : ARRAY,
                  'array_type' : [ INTEGER, STRING ], 
                  'optional' : True
                },
                { 'name' : 'hemisphere',
                  'type' : STRING,
                  'optional' : True,
                  'values' : [ 'right', 'left']
                },
                { 'name' : 'transgenic_lines',
                  'type' : ARRAY,
                  'array_type' : [ INTEGER,STRING ],
                  'optional' : True
                },
                { 'name' : 'injection_structures',
                  'type' : 'Array',
                  'array_type' : [ INTEGER,STRING ], 
                  'optional' : True
                },
                { 'name' : 'primary_structure_only',
                  'type' : BOOLEAN,
                  'optional' : True,
                },
                { 'name' : 'sort_order',
                  'type' : STRING,
                  'optional' : True,
                  'values' : [ 'asc', 'desc'],
                  'default' : 'desc'
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'mouse_connectivity_injection_coordinate' : {
            'parameters' : [
                { 'name': 'seed_point',
                  'type': ARRAY,
                  'array_type': FLOAT,
                  'optional' : False,
                },
                { 'name' : 'transgenic_lines',
                  'type' : ARRAY,
                  'array_type' : [ INTEGER, STRING ],
                  'optional' : True
                },
                { 'name' : 'injection_structures',
                  'type' : ARRAY,
                  'array_type' : [ INTEGER, STRING ],
                  'optional' : True
                },
                { 'name' : 'primary_structure_only',
                  'optional' : True
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'mouse_connectivity_injection_structure' : {
            'parameters' : [
                { 'name' : 'injection_structures',
                  'type' : ARRAY,
                  'array_type' : [ INTEGER, STRING ]
                },
                { 'name' : 'target_domain',
                  'type' : ARRAY,
                  'array_type' : [ INTEGER, STRING ],
                  'optional' : True
                },
                { 'name' : 'injection_hemisphere',
                  'type' : STRING,
                  'optional' : True,
                  'values' : [ 'right', 'left' ]
                },
                { 'name' : 'target_hemisphere',
                  'type' : STRING,
                  'optional' : True,
                  'values' : [ 'right', 'left' ]
                },
                { 'name' : 'transgenic_lines',
                  'type' : ARRAY,
                  'array_type' : [ INTEGER, STRING ],
                  'optional' : True
                },
                { 'name' : 'injection_domain',
                  'type' : ARRAY,
                  'array_type' : [ INTEGER, STRING ],
                  'optional' : True
                },
                { 'name' : 'primary_structure_only',
                  'type' : BOOLEAN,
                  'optional' : True
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'mouse_connectivity_target_spatial' : {
            'parameters' : [
                { 'name' : 'seed_point',
                  'type' : ARRAY,
                  'array_type' : FLOAT,
                },
                { 'name' : 'transgenic_lines',
                  'type' : ARRAY,
                  'array_type' : [ INTEGER, STRING ],
                  'optional' : True
                },
                { 'name' : 'section_data_set',
                  'type' : INTEGER,
                  'optional' : True 
                },
                { 'name' : 'injection_structures',
                  'type' : ARRAY,
                  'array_type' : [ INTEGER, STRING ],
                  'optional' : True
                },
                { 'name' : 'primary_structure_only',
                  'type' : BOOLEAN,
                  'optional' : True
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'nhp_lmd_microarray_correlation' : {
            'parameters' : [
                { 'name' : 'donors',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'structures',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                },
                { 'name' : 'probes',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'sort_order',
                  'type' : STRING,
                  'optional' : True,
                  'values' : [ 'asc', 'desc' ],
                  'default' : 'desc'
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'nhp_lmd_microarray_differential' : {
            'parameters' : [
                { 'name' : 'donors1',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'structures1',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'donors2',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'structures2',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'sort_by',
                  'type' : STRING,
                  'optional' : True,
                  'values' : [ 'p-value', 'fold-change' ],
                  'default' : 'p-value'
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'nhp_lmd_microarray_expression' : {
            'parameters' : [
                { 'name' : 'probes',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'nhp_macro_microarray_correlation' : {
            'parameters' : [
                { 'name' : 'donors',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'structures',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'probes',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'sort_order',
                  'type' : STRING,
                  'optional' : True,
                  'values' : [ 'asc', 'desc' ],
                  'default' : 'desc'
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'nhp_macro_microarray_differential' : {
            'parameters' : [
                { 'name' : 'donors1',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'structures1',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'donors2',
                  'type' : ARRAY,
                  'array_type' : INTEGER,
                  'optional' : True
                },
                { 'name' : 'structures2',
                  'array_type' : INTEGER,
                  'type' : ARRAY
                },
                { 'name' : 'sort_by',
                  'type' : STRING,
                  'optional' : True,
                  'values' : [ 'p-value', 'fold-change' ],
                  'default' : 'p-value'
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'nhp_macro_microarray_expression' : {
            'parameters' : [
                { 'name' : 'probes',
                  'type' : ARRAY,
                  'array_type' : INTEGER
                },
                { 'name' : 'start_row',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 0
                },
                { 'name' : 'num_rows',
                  'type' : INTEGER,
                  'optional' : True,
                  'default' : 2000
                }
            ]
        },
        'text_search' : {
            'parameters' : [
                { 'name' : 'query_string',
                  'type' : STRING
                },
                { 'name' : 'k',
                  'type' : STRING
                }
            ]
        }
    }



from allensdk.api.queries.ontologies_api import OntologiesApi

import unittest
from mock import MagicMock

class OntologiesApiTests(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(OntologiesApiTests, self).__init__(*args, **kwargs)
    
    
    def setUp(self):
        self.oa = OntologiesApi()
    
    
    def tearDown(self):
        self.oa = None
    
    
    def test_get_structure_graph(self):
        expected = "http://api.brain-map.org/api/v2/data/query.json?q=model::Structure,rma::criteria,[graph_id$in1],rma::options[num_rows$eq'all'][order$eqstructures.graph_order][count$eqfalse]"
        
        structure_graph_id = 1
        
        self.oa.json_msg_query = \
            MagicMock(name='json_msg_query')
        
        self.oa.get_structures(structure_graph_id)
        
        self.oa.json_msg_query.assert_called_once_with(expected)
    
    
    def test_list_structure_graphs(self):
        expected = "http://api.brain-map.org/api/v2/data/query.json?q=model::StructureGraph,rma::options[num_rows$eq'all'][count$eqfalse]"
        
        self.oa.json_msg_query = \
            MagicMock(name='json_msg_query')
               
        self.oa.get_structure_graphs()
        
        self.oa.json_msg_query.assert_called_once_with(expected)
        
    
    def test_list_structure_sets(self):
        expected = "http://api.brain-map.org/api/v2/data/query.json?q=model::StructureSet,rma::options[num_rows$eq'all'][count$eqfalse]"
        
        self.oa.json_msg_query = \
            MagicMock(name='json_msg_query')
               
        self.oa.get_structure_sets()
        
        self.oa.json_msg_query.assert_called_once_with(expected)


    def test_list_atlases(self):
        expected = "http://api.brain-map.org/api/v2/data/query.json?q=model::Atlas,rma::options[num_rows$eq'all'][count$eqfalse]"
        
        self.oa.json_msg_query = \
            MagicMock(name='json_msg_query')
               
        self.oa.get_atlases()
        
        self.oa.json_msg_query.assert_called_once_with(expected)

        
    def test_structure_graph_by_name(self):
        expected = u"http://api.brain-map.org/api/v2/data/query.json?q=model::Structure,rma::criteria,graph[structure_graphs.name$in'Mouse Brain Atlas'],rma::options[num_rows$eq'all'][order$eqstructures.graph_order][count$eqfalse]"
        
        self.oa.json_msg_query = \
            MagicMock(name='json_msg_query')
        
        self.oa.get_structures(structure_graph_names="'Mouse Brain Atlas'")
        
        self.oa.json_msg_query.assert_called_once_with(expected)


    def test_structure_graphs_by_names(self):
        expected = "http://api.brain-map.org/api/v2/data/query.json?q=model::Structure,rma::criteria,graph[structure_graphs.name$in'Mouse Brain Atlas','Human Brain Atlas'],rma::options[num_rows$eq'all'][order$eqstructures.graph_order][count$eqfalse]"
        
        self.oa.json_msg_query = \
            MagicMock(name='json_msg_query')
        
        self.oa.get_structures(structure_graph_names=["'Mouse Brain Atlas'",
                                                      "'Human Brain Atlas'"])
        
        self.oa.json_msg_query.assert_called_once_with(expected)

    
    def test_structure_set_by_id(self):
        expected = "http://api.brain-map.org/api/v2/data/query.json?q=model::Structure,rma::criteria,[structure_set_id$in8],rma::options[num_rows$eq'all'][order$eqstructures.graph_order][count$eqfalse]"
        
        self.oa.json_msg_query = \
            MagicMock(name='json_msg_query')
        
        self.oa.get_structures(structure_set_ids=8)
        
        self.oa.json_msg_query.assert_called_once_with(expected)
        
        
    def test_structure_sets_by_ids(self):
        expected = "http://api.brain-map.org/api/v2/data/query.json?q=model::Structure,rma::criteria,[structure_set_id$in7,8],rma::options[num_rows$eq'all'][order$eqstructures.graph_order][count$eqfalse]"
        
        self.oa.json_msg_query = \
            MagicMock(name='json_msg_query')
        
        self.oa.get_structures(structure_set_ids=[7,8])
        
        self.oa.json_msg_query.assert_called_once_with(expected)
        
        
    def test_structure_set_by_name(self):
        expected = "http://api.brain-map.org/api/v2/data/query.json?q=model::Structure,rma::criteria,structure_sets[name$in'Mouse Connectivity - Summary'],rma::options[num_rows$eq'all'][order$eqstructures.graph_order][count$eqfalse]"
        
        self.oa.json_msg_query = \
            MagicMock(name='json_msg_query')
        
        self.oa.get_structures(structure_set_names="'Mouse Connectivity - Summary'")
        
        self.oa.json_msg_query.assert_called_once_with(expected)


    def test_structure_set_by_names(self):
        expected = "http://api.brain-map.org/api/v2/data/query.json?q=model::Structure,rma::criteria,structure_sets[name$in'NHP - Coarse','Mouse Connectivity - Summary'],rma::options[num_rows$eq'all'][order$eqstructures.graph_order][count$eqfalse]"
        
        self.oa.json_msg_query = \
            MagicMock(name='json_msg_query')
        
        self.oa.get_structures(structure_set_names=["'NHP - Coarse'",
                                                    "'Mouse Connectivity - Summary'"])
        
        self.oa.json_msg_query.assert_called_once_with(expected)


    def test_structure_set_no_order(self):
        expected = "http://api.brain-map.org/api/v2/data/query.json?q=model::Structure,rma::criteria,[graph_id$in1],rma::options[num_rows$eq'all'][count$eqfalse]"
        
        self.oa.json_msg_query = \
            MagicMock(name='json_msg_query')
        
        self.oa.get_structures(1, order=None)
        
        self.oa.json_msg_query.assert_called_once_with(expected)


    def test_atlas_1(self):
        expected = "http://api.brain-map.org/api/v2/data/query.json?q=model::Atlas,rma::criteria,[id$in1],structure_graph(ontology),graphic_group_labels,rma::include,structure_graph(ontology),graphic_group_labels,rma::options[only$eq'atlases.id,atlases.name,atlases.image_type,ontologies.id,ontologies.name,structure_graphs.id,structure_graphs.name,graphic_group_labels.id,graphic_group_labels.name'][num_rows$eq'all'][count$eqfalse]"
        
        atlas_id = 1
        
        self.oa.json_msg_query = \
            MagicMock(name='json_msg_query')
        
        self.oa.get_atlases_table(atlas_id)
        
        self.oa.json_msg_query.assert_called_once_with(expected)
    
    
    def test_atlas_verbose(self):
        expected = "http://api.brain-map.org/api/v2/data/query.json?q=model::Atlas,rma::criteria,structure_graph(ontology),graphic_group_labels,rma::include,structure_graph(ontology),graphic_group_labels,rma::options[num_rows$eq'all'][count$eqfalse]"
        
        self.oa.json_msg_query = \
            MagicMock(name='json_msg_query')
        
        self.oa.get_atlases_table(brief=False)
        
        self.oa.json_msg_query.assert_called_once_with(expected)




import unittest
from mock import MagicMock
from allensdk.api.queries.synchronization_api import SynchronizationApi

class SynchronizationApiTests(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(SynchronizationApiTests, self).__init__(*args, **kwargs)
        self.ida = None
    
    
    def setUp(self):
        self.sa = SynchronizationApi()
    
    
    def tearDown(self):
        self.ida = None
    
    
    def test_image_to_image(self):
        '''
        Notes
        -----
        Expected link is slightly modified for json and float serialization of zeros.
        
        See: `Image Alignment `<http://help.brain-map.org/display/mouseconnectivity/API#API-ImageAlignment>_
        , link labeled 'Sync a VISp and VISal experiment to a location in a SCs SectionDataSet'.
        '''
        expected = 'http://api.brain-map.org/api/v2/image_to_image/114754496.json?x=18232.000000&y=10704.000000&section_data_set_ids=113887162,116903968'

        self.sa.json_msg_query = \
            MagicMock(name='json_msg_query')
        
        section_image_id = 114754496
        (x, y) = (18232, 10704)
        section_data_set_ids = [113887162, 116903968]
        
        _ = self.sa.get_image_to_image(section_image_id,
                                       x, y,
                                       section_data_set_ids)
        
        self.sa.json_msg_query.assert_called_once_with(expected)


    def test_image_to_image_2d(self):
        expected = 'http://api.brain-map.org/api/v2/image_to_image_2d/68173101.json?x=6208.000000&y=2368.000000&section_image_ids=68173103,68173105,68173107'
                    
        self.sa.json_msg_query = \
            MagicMock(name='json_msg_query')
        
        section_image_id = 68173101
        (x, y) = (6208, 2368)
        section_image_ids = [68173103, 68173105, 68173107]
        
        _ = self.sa.get_image_to_image_2d(section_image_id,
                                          x, y,
                                          section_image_ids)
        
        self.sa.json_msg_query.assert_called_once_with(expected)
    

    def test_reference_to_image(self):
        expected = 'http://api.brain-map.org/api/v2/reference_to_image/10.json?x=6085.000000&y=3670.000000&z=4883.000000&section_data_set_ids=68545324,67810540'
                    
        self.sa.json_msg_query = \
            MagicMock(name='json_msg_query')
        
        reference_space_id = 10
        (x, y, z) = (6085, 3670, 4883)
        section_data_set_ids = [68545324, 67810540]
        
        _ = self.sa.get_reference_to_image(reference_space_id,
                                          x, y, z,
                                          section_data_set_ids)
        
        self.sa.json_msg_query.assert_called_once_with(expected)


    def test_image_to_reference(self):
        expected = 'http://api.brain-map.org/api/v2/image_to_reference/68173101.json?x=6208.000000&y=2368.000000'
                    
        self.sa.json_msg_query = \
            MagicMock(name='json_msg_query')
        
        section_image_id = 68173101
        (x, y) = (6208, 2368)
        
        _ = self.sa.get_image_to_reference(section_image_id,
                                          x, y)
        
        self.sa.json_msg_query.assert_called_once_with(expected)


    def test_structure_to_image(self):
        expected = 'http://api.brain-map.org/api/v2/structure_to_image/68545324.json?structure_ids=315,698,1089,703,477,803,512,549,1097,313,771,354'
                    
        self.sa.json_msg_query = \
            MagicMock(name='json_msg_query')
        
        section_data_set_id = 68545324
        structure_ids = [315,698,1089,703,477,803,512,549,1097,313,771,354]
        
        _ = self.sa.get_structure_to_image(section_data_set_id,
                                           structure_ids)
        
        self.sa.json_msg_query.assert_called_once_with(expected)

    
    def test_image_to_atlas(self):
        '''
        Notes
        -----
        Expected link is slightly modified for json and float serialization of zeros.
        
        See: `Image Alignment `<http://help.brain-map.org/display/mouseconnectivity/API#API-ImageAlignment>_
        , link labeled 'Sync the P56 coronal reference atlas to a location in the SCs SectionDataSet'.
        '''
        expected = 'http://api.brain-map.org/api/v2/image_to_atlas/114754496.json?x=18232.000000&y=10704.000000&atlas_id=1'

        self.sa.json_msg_query = \
            MagicMock(name='json_msg_query')
        
        section_image_id = 114754496
        (x, y) = (18232, 10704)
        atlas_id = 1
        _ = self.sa.get_image_to_atlas(section_image_id,
                                       x, y,
                                       atlas_id)
        
        self.sa.json_msg_query.assert_called_once_with(expected)




import unittest
from mock import MagicMock
import allensdk.core.json_utilities as ju
from allensdk.api.queries.rma_template import RmaTemplate

class ApiTemplateTests(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(ApiTemplateTests, self).__init__(*args, **kwargs)
        self.templates = None
    
    
    def setUp(self):
        self.templates = \
            {"ontology_queries": [
                {'name': 'structures_by_graph_ids',
                 'description': 'see name',
                 'model': 'Structure',
                 'criteria': '[graph_id$in{{ graph_ids }}]',
                 'order': ['structures.graph_order'],
                 'num_rows': 'all',
                 'count': False,
                 'criteria_params': ['graph_ids']
                },
                {'name': 'structures_by_graph_names',
                 'description': 'see name',
                 'model': 'Structure',
                 'criteria': 'graph[structure_graphs.name$in{{ graph_names }}]',
                 'order': ['structures.graph_order'],
                 'num_rows': 'all',
                 'count': False,
                 'criteria_params': ['graph_names']
                },
                {'name': 'structures_by_set_ids',
                 'description': 'see name',
                 'model': 'Structure',
                 'criteria': '[structure_set_id$in{{ set_ids }}]',
                 'order': ['structures.graph_order'],
                 'num_rows': 'all',
                 'count': False,
                 'criteria_params': ['set_ids']
                },
                {'name': 'structures_by_set_names',
                 'description': 'see name',
                 'model': 'Structure',
                 'criteria': 'structure_sets[name$in{{ set_names }}]',
                 'order': ['structures.graph_order'],
                 'num_rows': 'all',
                 'count': False,
                 'criteria_params': ['set_names']
                },
                {'name': 'structure_graphs_list',
                 'description': 'see name',
                 'model': 'StructureGraph',
                 'num_rows': 'all',
                 'count': False
                },
                {'name': 'structure_sets_list',
                 'description': 'see name',
                 'model': 'StructureSet',
                 'num_rows': 'all',
                 'count': False
                },
                {'name': 'atlases_list',
                 'description': 'see name',
                 'model': 'Atlas',
                 'num_rows': 'all',
                 'count': False
                },
                {'name': 'atlases_table',
                 'description': 'see name',
                 'model': 'Atlas',
                 'criteria': '{% if graph_ids is defined %}[graph_id$in{{ graph_ids }}],{% endif %}structure_graph(ontology),graphic_group_labels',
                 'include': '[structure_graph(ontology),graphic_group_labels',
                 'num_rows': 'all',
                 'count': False,
                 'criteria_params': ['graph_ids']
                },
                {'name': 'atlases_table_brief',
                 'description': 'see name',
                 'model': 'Atlas',
                 'criteria': 'structure_graph(ontology),graphic_group_labels',
                 'include': 'structure_graph(ontology),graphic_group_labels',
                 'only': ['atlases.id',
                           'atlases.name',
                           'atlases.image_type',
                           'ontologies.id',
                           'ontologies.name',
                           'structure_graphs.id',
                           'structure_graphs.name',
                           'graphic_group_labels.id',
                           'graphic_group_labels.name'],
                 'num_rows': 'all',
                 'count': False
                }
            ]}
        self.rma = RmaTemplate(query_manifest=self.templates)
    
    
    def tearDown(self):
        self.rma= None
        self.templates = None
        
    
    def test_atlases_list(self):
        ju.read_url_get = \
            MagicMock(name='read_url_get',
                      return_value={ 'msg': [{ 'whatever': True }] })
        
        self.rma.template_query('ontology_queries',
                                'atlases_list')
        
        ju.read_url_get.assert_called_once_with('http://api.brain-map.org/api/v2/data/query.json?q=model::Atlas,rma::options%5Bnum_rows$eq%27all%27%5D%5Bcount$eqfalse%5D')
    
    
    def test_structure_graphs_list(self):
        ju.read_url_get = \
            MagicMock(name='read_url_get',
                      return_value={ 'msg': [{ 'whatever': True }] })
        
        self.rma.template_query('ontology_queries',
                                'structure_graphs_list')
        
        ju.read_url_get.assert_called_once_with('http://api.brain-map.org/api/v2/data/query.json?q=model::StructureGraph,rma::options%5Bnum_rows$eq%27all%27%5D%5Bcount$eqfalse%5D')
    
    
    def test_structure_sets_list(self):
        ju.read_url_get = \
            MagicMock(name='read_url_get',
                      return_value={ 'msg': [{ 'whatever': True }] })
        
        self.rma.template_query('ontology_queries',
                                'structure_sets_list')
        
        ju.read_url_get.assert_called_once_with('http://api.brain-map.org/api/v2/data/query.json?q=model::StructureSet,rma::options%5Bnum_rows$eq%27all%27%5D%5Bcount$eqfalse%5D')
    
    
    def test_structures_by_graph_ids(self):
        ju.read_url_get = \
            MagicMock(name='read_url_get',
                      return_value={ 'msg': [{ 'whatever': True }] })
        
        self.rma.template_query('ontology_queries',
                                'structures_by_graph_ids',
                                graph_ids='1')
        
        ju.read_url_get.assert_called_once_with('http://api.brain-map.org/api/v2/data/query.json?q=model::Structure,rma::criteria,%5Bgraph_id$in1%5D,rma::options%5Bnum_rows$eq%27all%27%5D%5Border$eqstructures.graph_order%5D%5Bcount$eqfalse%5D')
    
    
    def test_structures_by_two_graph_ids(self):
        ju.read_url_get = \
            MagicMock(name='read_url_get',
                      return_value={ 'msg': [{ 'whatever': True }] })
        
        self.rma.template_query('ontology_queries',
                                'structures_by_graph_ids',
                                graph_ids=[1, 2])
        
        ju.read_url_get.assert_called_once_with('http://api.brain-map.org/api/v2/data/query.json?q=model::Structure,rma::criteria,%5Bgraph_id$in1,2%5D,rma::options%5Bnum_rows$eq%27all%27%5D%5Border$eqstructures.graph_order%5D%5Bcount$eqfalse%5D')
    
    
    def test_structures_by_graph_names(self):
        ju.read_url_get = \
            MagicMock(name='read_url_get',
                      return_value={ 'msg': [{ 'whatever': True }] })
        
        self.rma.template_query('ontology_queries',
                                'structures_by_graph_names',
                                graph_names=self.rma.quote_string('Human+Brain+Atlas'))
        
        ju.read_url_get.assert_called_once_with('http://api.brain-map.org/api/v2/data/query.json?q=model::Structure,rma::criteria,graph%5Bstructure_graphs.name$in%27Human+Brain+Atlas%27%5D,rma::options%5Bnum_rows$eq%27all%27%5D%5Border$eqstructures.graph_order%5D%5Bcount$eqfalse%5D')
    
    
    def test_structures_by_set_ids(self):
        ju.read_url_get = \
            MagicMock(name='read_url_get',
                      return_value={ 'msg': [{ 'whatever': True }] })
        
        self.rma.template_query('ontology_queries',
                                'structures_by_graph_ids',
                                graph_ids='1')
        
        ju.read_url_get.assert_called_once_with('http://api.brain-map.org/api/v2/data/query.json?q=model::Structure,rma::criteria,%5Bgraph_id$in1%5D,rma::options%5Bnum_rows$eq%27all%27%5D%5Border$eqstructures.graph_order%5D%5Bcount$eqfalse%5D')
    
    
    def test_atlases_table(self):
        ju.read_url_get = \
            MagicMock(name='read_url_get',
                      return_value={ 'msg': [{ 'whatever': True }] })
        
        self.rma.template_query('ontology_queries',
                                'atlases_table')
        
        ju.read_url_get.assert_called_once_with('http://api.brain-map.org/api/v2/data/query.json?q=model::Atlas,rma::criteria,structure_graph%28ontology%29,graphic_group_labels,rma::include,%5Bstructure_graph%28ontology%29,graphic_group_labels,rma::options%5Bnum_rows$eq%27all%27%5D%5Bcount$eqfalse%5D')
    
    
    def test_atlases_table_one_graph(self):
        ju.read_url_get = \
            MagicMock(name='read_url_get',
                      return_value={ 'msg': [{ 'whatever': True }] })
        
        self.rma.template_query('ontology_queries',
                                'atlases_table',
                                graph_ids=1)
        
        ju.read_url_get.assert_called_once_with('http://api.brain-map.org/api/v2/data/query.json?q=model::Atlas,rma::criteria,%5Bgraph_id$in1%5D,structure_graph%28ontology%29,graphic_group_labels,rma::include,%5Bstructure_graph%28ontology%29,graphic_group_labels,rma::options%5Bnum_rows$eq%27all%27%5D%5Bcount$eqfalse%5D')
    
    
    def test_atlases_table_brief(self):
        ju.read_url_get = \
            MagicMock(name='read_url_get',
                      return_value={ 'msg': [{ 'whatever': True }] })
        
        self.rma.template_query('ontology_queries',
                                'atlases_table_brief')
        
        ju.read_url_get.assert_called_once_with('http://api.brain-map.org/api/v2/data/query.json?q=model::Atlas,rma::criteria,structure_graph%28ontology%29,graphic_group_labels,rma::include,structure_graph%28ontology%29,graphic_group_labels,rma::options%5Bonly$eq%27atlases.id,atlases.name,atlases.image_type,ontologies.id,ontologies.name,structure_graphs.id,structure_graphs.name,graphic_group_labels.id,graphic_group_labels.name%27%5D%5Bnum_rows$eq%27all%27%5D%5Bcount$eqfalse%5D')



import unittest
from mock import MagicMock
from allensdk.api.queries.image_download_api import ImageDownloadApi

class ImageDownloadApiTests(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(ImageDownloadApiTests, self).__init__(*args, **kwargs)
        self.ida = None
    
    
    def setUp(self):
        self.ida = ImageDownloadApi()
    
    
    def tearDown(self):
        self.ida = None
    
    
    def test_api_doc_url_download_image_downsampled(self):
        '''
        Notes
        -----
        See: `Experimental Overview and Metadata `<http://help.brain-map.org/display/mouseconnectivity/API#API-ExperimentalOverviewandMetadata>_
        , link labeled 'Download image downsampled by factor of 6 using default thresholds'.
        '''
        expected = 'http://api.brain-map.org/api/v2/section_image_download/126862575?downsample=6&range=0,932,0,1279,0,4095'
        path = '126862575.jpg'
       
        self.ida.retrieve_file_over_http = \
            MagicMock(name='retrieve_file_over_http')
        
        section_image_id = 126862575
        self.ida.download_section_image(section_image_id,
                                        downsample=6,
                                        range=[0,932,  0,1279, 0,4095])
        
        self.ida.retrieve_file_over_http.assert_called_once_with(expected, path)
    
    
    def test_api_doc_url_download_image_full_resolution(self):
        '''
        Notes
        -----
        See: `Experimental Overview and Metadata `<http://help.brain-map.org/display/mouseconnectivity/API#API-ExperimentalOverviewandMetadata>_
        , link labeled 'Download a region of interest at full resolution using default thresholds'.
        '''
        expected = 'http://api.brain-map.org/api/v2/section_image_download/126862575?left=19045&top=11684&width=1000&height=1000&range=0,932,0,1279,0,4095'
        path = '126862575.jpg'

        self.ida.retrieve_file_over_http = \
            MagicMock(name='retrieve_file_over_http')
        
        section_image_id = 126862575
        self.ida.download_section_image(section_image_id,
                                        left=19045,
                                        top=11684,
                                        width=1000,
                                        height=1000,
                                        range=[0,932, 0,1279, 0,4095])
        
        self.ida.retrieve_file_over_http.assert_called_once_with(expected, path)
